//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw.db.server;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Version;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Implementierung f&uuml;r einen Datensatz in der Versionstabelle.
 * 
 * <pre>
 * CREATE TABLE version (
 *  id NUMERIC default UNIQUEKEY('version'),
 *  name varchar(255) NOT NULL,
 *  version int(5) NOT NULL,
 *  UNIQUE (id),
 *  PRIMARY KEY (id)
 * );
 * </pre>
 * 
 * Getters und Setters f&uuml;r den Primary Key (id) sind nicht n&ouml;tig. Jede
 * Methode muss zwingend eine RemoteException werfen. <br>
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class VersionImpl extends AbstractDBObject implements Version {

	/**
	 * serialVersionUID
	 */
	private static final long serialVersionUID = -7949815027870224311L;
	
	/**
	 * Unser &Uuml;bersetzer
	 */
	private static final I18N I18N = Settings.getI18n();

	/**
	 * Standart Konstruktor
	 * 
	 * @throws RemoteException
	 */
	public VersionImpl() throws RemoteException {
		super();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return "name";
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
	 */
	@Override
	protected String getTableName() {
		return "version";
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Version#getName()
	 */
	@Override
	public String getName() throws RemoteException {
		return (String) getAttribute("name");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Version#getVersion()
	 */
	@Override
	public int getVersion() throws RemoteException {
		// Wir fangen bei 0 an mit dem Zaehlen
		Integer version;
		try {
			version = (Integer) getAttribute("version");
			if (version == null) {
				version = 0; // NOPMD 
			}
		} catch (RemoteException e) {
			version = 0;
		}
		return version.intValue();
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Version#setVersion(int)
	 */
	@Override
	public void setVersion(final int newVersion) throws RemoteException {
		if (newVersion < 0) {
			throw new RemoteException(I18N.tr("version cannot be smaller than zero"));
		}
		setAttribute("version", Integer.valueOf(newVersion));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Version#setName(java.lang.String)
	 */
	@Override
	public void setName(final String name) throws RemoteException {
		setAttribute("name", name);
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
	 */
	@Override
	protected void insertCheck() throws ApplicationException {
		

		try {
			if (getName() == null || getName().length() == 0) {
				throw new ApplicationException(
						I18N.tr("No name for Version given."));
			}
		} catch (RemoteException re) {
			Logger.error("error while checking version", re);
			throw new ApplicationException(
					I18N.tr("Error while checking the version"), re);
		}
		super.insertCheck();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
	 */
	@Override
	protected void updateCheck() throws ApplicationException {
		insertCheck();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insert()
	 */
	@Override
	public void insert() throws RemoteException, ApplicationException {
		setVersion(getVersion()); // speichert automatisch die Startnummer
		super.insert();
	}

}

