package de.frankmuenster.jameica.orverw.gui.input;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FontDialog;

import de.frankmuenster.jameica.orverw.OrdnerVerwaltung;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.AbstractInput;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * @author munstfr
 * 
 */
@SuppressWarnings({ "PMD.GodClass" })
public class TextStyleInput extends AbstractInput implements SelectionListener {

	/**
	 * Unser &Uuml;bersetzer
	 */
	private final static I18N I18N = Application.getPluginLoader()
			.getPlugin(OrdnerVerwaltung.class).getResources().getI18N();

	/**
	 * Show foreground color button
	 */
	public static final int FOREGROUND = 1;
	/**
	 * Show background color button
	 */
	public static final int BACKGROUND = 2;
	/**
	 * Show alignment buttons
	 */
	public static final int ALIGNMENT = 4;

	/**
	 * Der R&uuml<gabewewrt
	 */
	private TextStyle value;

	/**
	 * Der SWT Kontainer f&uuml;r das Input
	 */
	private transient Composite container;

	/**
	 * Das Beispieltext Label
	 */
	private transient CLabel example;

	/**
	 * Die Breite des Beispieltextfeldes
	 */
	private transient int exampleWidth;

	/**
	 * Der Button zum Anzeigen des Schriftauswahldialoges
	 */
	private transient Button font;

	/**
	 * Der Button zur Anzeigen des Schriftfarbendialoges
	 */
	private transient Button textColor;

	/**
	 * Der Button zur Anzeigen des Hintergrunffarbendialoges
	 */
	private transient Button backgroundColor;

	/**
	 * Der Text-Links-Ausrichten-Button
	 */
	private transient Button left;

	/**
	 * Der Text-Zentrieren-Button
	 */
	private transient Button center;

	/**
	 * Der Text-Rechts-Ausrichten-Button
	 */
	private transient Button right;

	/**
	 * Sichtbarkeit des Textfarben-Button
	 */
	private transient boolean showFgButton;

	/**
	 * Sichtbarkeit des Hintergrundfarben-Button
	 */
	private transient boolean showBgButton;

	/**
	 * Sichtbarkeit des Textausrichtungs-Buttons
	 */
	private transient boolean showAlignButtons;

	/**
	 * Haben wir den Fokus?
	 */
	private transient boolean hasFocus;

	/**
	 * Sind wir aktiviert?
	 */
	private boolean enabled;

	/**
	 * Erzeugt ein TextStyleInput Part mit allen Buttons sichtbar.
	 * 
	 * @param style
	 *            das TextStyle Object, dass als Grundlage dient.
	 * @throws ApplicationException
	 */
	public TextStyleInput(final TextStyle style) throws ApplicationException {
		this(style, FOREGROUND | BACKGROUND | ALIGNMENT, new Point(200, 0));
	}

	/**
	 * Erzeugt ein TextStyleInput Part mit allen Buttons sichtbar.
	 * 
	 * @param style
	 *            das TextStyle Object, dass als Grundlage dient.
	 * @param exampleWidth
	 *            Breite des Beispielfeldes
	 * @throws ApplicationException
	 */
	public TextStyleInput(final TextStyle style, final Point exampleWidth)
			throws ApplicationException {
		this(style, FOREGROUND | BACKGROUND | ALIGNMENT, exampleWidth);
	}

	/**
	 * Erzeugt ein TextStyleInput Part mit den angegebenen Buttons.
	 * 
	 * @param style
	 *            das TextStyle Object, dass als Grundlage dient.
	 * @param showOption
	 *            Buttons die sichtbar sind. M&ouml;gliche Werte sind
	 *            TextStyleInput.FOREGROUND, TextStyleInput.BACKGROUND,
	 *            TextStyleInput.ALIGNMENT.
	 * @throws ApplicationException
	 */
	public TextStyleInput(final TextStyle style, final int showOption)
			throws ApplicationException {

		this(style, showOption, new Point(200, 0));

	}

	/**
	 * Erzeugt ein TextStyleInput Part mit den angegebenen Buttons.
	 * 
	 * @param style
	 *            das TextStyle Object, dass als Grundlage dient.
	 * @param showOption
	 *            Buttons die sichtbar sind. M&ouml;gliche Werte sind
	 *            TextStyleInput.FOREGROUND, TextStyleInput.BACKGROUND,
	 *            TextStyleInput.ALIGNMENT.
	 * @param exampleWidth
	 *            Breite des Beispielfeldes
	 * @throws ApplicationException
	 */
	public TextStyleInput(final TextStyle style, final int showOption,
			final Point exampleWidth) throws ApplicationException {

		super();

		if (style == null) {
			new ApplicationException("TextStyle cannot be NULL");
		}
		this.value = style;
		this.showFgButton = (showOption & FOREGROUND) == FOREGROUND;
		this.showBgButton = (showOption & BACKGROUND) == BACKGROUND;
		this.showAlignButtons = (showOption & ALIGNMENT) == ALIGNMENT;
		this.exampleWidth = exampleWidth.x;

	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#getValue()
	 */
	@Override
	public Object getValue() {
		return this.value;

	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#setValue(java.lang.Object)
	 */
	@Override
	public void setValue(final Object value) {
		if (!(value instanceof TextStyle)) {
			return;
		}

		this.value = (TextStyle) value;
		if (!showAlignButtons) {
			this.value.setAlignment(SWT.LEFT);
		}

		if (this.container != null && !this.container.isDisposed()) {
			redraw();
		}

	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#getControl()
	 */
	@Override
	public Control getControl() {

		if (container != null) {
			return container; // NOPMD
		}

		container = new Composite(getParent(), SWT.NONE);
		initContainer();
		createExample();
		createButtons();

		if (this.hasFocus) {
			container.setFocus();
		}
		return container;
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#focus()
	 */
	@Override
	public void focus() {
		this.hasFocus = true;
		if (this.container != null && !this.container.isDisposed()) {
			container.setFocus();
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#disable()
	 */
	@Override
	public void disable() {
		setEnabled(false);
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#enable()
	 */
	@Override
	public void enable() {
		setEnabled(true);
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(final boolean enabled) {
		this.enabled = enabled;
		if (container != null && !container.isDisposed()) {
			container.setEnabled(enabled);
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#isEnabled()
	 */
	@Override
	public boolean isEnabled() {
		Boolean enabled;
		if (container == null || container.isDisposed()) {
			enabled = this.enabled;
		} else {
			enabled = container.getEnabled();
		}
		return enabled;
	}

	private void redraw() {

		// Font
		example.setFont(value.getFont());
		example.setForeground(value.getTextColor());
		example.setBackground(value.getBackgroundColor());
		example.setAlignment(this.value.getAlignment());
		if (left != null && center != null && right != null) {
			switch (this.value.getAlignment()) {
			case SWT.LEFT:
				left.setSelection(true);
				center.setSelection(false);
				right.setSelection(false);
				break;
			case SWT.CENTER:
				left.setSelection(false);
				center.setSelection(true);
				right.setSelection(false);
				break;
			case SWT.RIGHT:
				left.setSelection(false);
				center.setSelection(false);
				right.setSelection(true);
				break;
			default:
				break;
			}
		}

		// Layout anpassen
		GUI.getCurrentView().getParent().layout();

	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse
	 *      .swt.events.SelectionEvent)
	 */
	@Override
	public void widgetDefaultSelected(final SelectionEvent arg0) {
		// Nothing to do
	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt
	 *      .events.SelectionEvent)
	 */
	@Override
	public void widgetSelected(final SelectionEvent event) {

		final Object source = event.getSource();
		if (source instanceof Button) {
			handleButtonClicked((Button) source);
			redraw();
		}

	}

	@SuppressWarnings({ "PMD.ModifiedCyclomaticComplexity",
			"PMD.CyclomaticComplexity", "PMD.StdCyclomaticComplexity" })
	private void handleButtonClicked(final Button button) {

		if (button.equals(font)) {
			final FontDialog fontDialog = new FontDialog(GUI.getShell());
			fontDialog.setEffectsVisible(false);
			fontDialog.setFontList(this.value.getFont().getFontData());
			final FontData fontData = fontDialog.open();
			if (fontData != null) {
				final Font font = new Font(GUI.getDisplay(), fontData);
				this.value.setFont(font);
			}
		} else if (button.equals(textColor)) {
			final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
			colorDialog.setRGB(this.value.getTextColor().getRGB());
			final RGB colorRGB = colorDialog.open();
			if (colorRGB != null) {
				final Color color = new Color(GUI.getDisplay(), colorRGB);
				this.value.setTextColor(color);
			}
		} else if (button.equals(backgroundColor)) {
			final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
			colorDialog.setRGB(this.value.getBackgroundColor().getRGB());
			final RGB colorRGB = colorDialog.open();
			if (colorRGB != null) {
				final Color color = new Color(GUI.getDisplay(), colorRGB);
				this.value.setBackgroundColor(color);
			}
		} else if (button.equals(left)) {
			this.value.setAlignment(SWT.LEFT);
		} else if (button.equals(center)) {
			this.value.setAlignment(SWT.CENTER);
		} else if (button.equals(right)) {
			this.value.setAlignment(SWT.RIGHT);
		}

		redraw();

	}

	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	private void initContainer() {
		int colums = 7;
		if (!showFgButton) {
			colums--;
		}
		if (!showAlignButtons) {
			colums -= 3;
		}
		if (!showBgButton) {
			colums--;
		}
		container.setLayout(new GridLayout(colums, false));

	}

	private void createExample() {
		example = new CLabel(container, SWT.NONE);
		final GridData gridData = new GridData();
		gridData.widthHint = exampleWidth;
		gridData.minimumWidth = 40;
		example.setLayoutData(gridData);
		example.setTopMargin(4);
		example.setBottomMargin(4);
		example.setText(" Beispieltext ");
		example.setFont(value.getFont());
		example.setForeground(value.getTextColor());
		example.setBackground(value.getBackgroundColor());
		example.setAlignment(value.getAlignment());
	}

	private void createButtons() {
		font = new Button(container, SWT.PUSH);
		font.setText(I18N.tr("Font"));
		font.addSelectionListener(this);

		if (showFgButton) {
			textColor = new Button(container, SWT.PUSH);
			textColor.setImage(SWTUtil.getImage("colors-icon.png"));
			textColor.addSelectionListener(this);
			textColor.setToolTipText(I18N.tr("Text Color"));
		}

		if (showAlignButtons) {
			left = new Button(container, SWT.TOGGLE | SWT.CENTER);
			left.setImage(SWTUtil.getImage("align_left.png"));
			left.setSelection(true);
			left.addSelectionListener(this);

			center = new Button(container, SWT.TOGGLE | SWT.CENTER);
			center.setImage(SWTUtil.getImage("align_center.png"));
			center.addSelectionListener(this);

			right = new Button(container, SWT.TOGGLE | SWT.CENTER);
			right.setImage(SWTUtil.getImage("align_right.png"));
			right.addSelectionListener(this);
		}

		if (showBgButton) {
			backgroundColor = new Button(container, SWT.PUSH);
			backgroundColor.setImage(SWTUtil.getImage("colors-icon.png"));
			backgroundColor.addSelectionListener(this);
			backgroundColor.setToolTipText(I18N.tr("Background Color"));

		}
	}
}
