/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts.labelpart;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import de.willuhn.jameica.gui.Part;

/**
 * Ein Texteingabefeld bei dem die Gr&ouml;&szlig;e, der Schrifttyp und Farben
 * angegeben werden k&ouml;nnen. Dazu gibt es noch einen Listener, der
 * benachrichtigt, wenn der Text ver&auml;ndert wurde.
 * 
 * @author munstfr
 * 
 */
public class LabelInputText implements Part {

	/**
	 * Das Standartabstand
	 */
	private static final int INDENT = 15;

	/**
	 * Das eigentliche Eingabe Widget
	 */
	private transient StyledText inputText;
	/**
	 * Der Inhalt des Eingabefeldes. Wird bentigt um nderungen zu sehen.
	 */
	private String text;
	/**
	 * Die Gre des Eingabefeldes
	 */
	private Point size;
	/**
	 * Der Schrifttyp des Textes
	 */
	private Font font;
	/**
	 * Die Textfarbe
	 */
	private Color foreground;
	/**
	 * Die Hintergrundfarbe
	 */
	private Color background;
	/**
	 * Die Textausrichtung
	 */
	private int textAlignment;

	/**
	 * Die Listener fr einen Button-Click-Event
	 */
	private final transient Set<LabelInputTextChangeListener> listeners = new CopyOnWriteArraySet<>();

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public final void addListener(final LabelInputTextChangeListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public final void removeListener(final LabelInputTextChangeListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten
	 * Buttons
	 * 
	 * @param methode
	 */
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	public final void notifyListeners() {
		for (final LabelInputTextChangeListener listener : listeners) {
			listener.notifytextChanged();
		}
	}

	LabelInputText(final Point size) {
		this(size, SWT.LEFT);
	}

	LabelInputText(final Point size, final int textAlignment) {

		this.size = size;
		this.textAlignment = textAlignment;

	}

	/**
	 * @param parent
	 * @return inputText
	 */
	public void paint(final Composite parent) {

		if (inputText != null) {
			return;
		}

		inputText = new StyledText(parent, SWT.NONE);
		inputText.addKeyListener(new MyKeyListener());
		inputText.addFocusListener(new MyFocusListener());

		// GridLayout
		final GridData gridData = new GridData();
		gridData.heightHint = size.y;
		gridData.widthHint = size.x - 2 * INDENT;
		inputText.setLayoutData(gridData);

		inputText.setLeftMargin(INDENT);
		inputText.setRightMargin(INDENT);
		inputText.setWordWrap(true);
		// inputText.setWrapIndent(indent);
		inputText.setAlignment(getTextAlignment());

		if (text != null) {
			inputText.setText(text);
		}
		if (font != null) {
			inputText.setFont(font);
		}
		if (foreground != null) {
			inputText.setForeground(foreground);
		}
		if (background != null) {
			inputText.setBackground(background);
		}

	}

	/**
	 * @return size
	 */
	public Point getSize() {
		return size;
	}

	/**
	 * @param size
	 *            das zu setzende Objekt size
	 */
	public void setSize(final Point size) {
		if (size != null) {
			this.size = size;
		}
	}

	/**
	 * @return text
	 */
	public String getText() {
		return this.text;
	}

	/**
	 * @return textAlignment
	 */
	public int getTextAlignment() {
		return textAlignment;
	}

	/**
	 * @param text
	 *            das zu setzende Objekt text
	 */
	public void setText(final String text) {
		if (text != null) {
			this.text = text;
			if (inputText != null) {
				inputText.setText(text);
			}
		}
	}

	/**
	 * @return fontColor
	 */
	public Color getForeground() {
		return inputText.getForeground();
	}

	/**
	 * @param color
	 *            das zu setzende Objekt fontColor
	 */
	public void setForeground(final Color color) {
		if (color != null) {
			this.foreground = color;
			if (inputText != null) {
				inputText.setForeground(color);
			}
		}
	}

	/**
	 * @return font
	 */
	public Font getFont() {
		return inputText.getFont();
	}

	/**
	 * @param font
	 *            das zu setzende Objekt font
	 */
	public void setFont(final Font font) {
		if (font != null) {
			this.font = font;
			if (inputText != null) {
				inputText.setFont(font);
			}
		}
	}

	/**
	 * @return background
	 */
	public Color getBackground() {
		return inputText.getBackground();
	}

	/**
	 * @param background
	 *            das zu setzende Objekt background
	 */
	public void setBackground(final Color background) {
		if (background != null) {
			this.background = background;
			if (inputText != null) {
				inputText.setBackground(background);
			}
		}
	}

	/**
	 * @param textAlignment
	 *            die zu setzende Textausrichtung
	 */
	public void setTextAlignment(final int textAlignment) {

		this.textAlignment = textAlignment;
		if (inputText != null) {
			inputText.setAlignment(textAlignment);
		}
	}

	/**
	 * Privater KeyListener
	 * 
	 * @author munstfr
	 * @version
	 */
	@SuppressWarnings("PMD.AtLeastOneConstructor")
	private class MyKeyListener implements KeyListener {

		/**
		 * @see org.eclipse.swt.events.KeyListener#keyReleased(org.eclipse.swt.events.KeyEvent)
		 */
		@Override
		public void keyReleased(final KeyEvent arg0) {
			// nothing to do
		}

		/**
		 * @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
		 */
		@Override
		public void keyPressed(final KeyEvent arg0) {
			// System.out.println("Keypressed in Textinput " + arg0.keyCode);
			if (!text.equals(inputText.getText())) {
				notifyListeners();
			}
		}
	}

	/**
	 * Privater FocusListener
	 * 
	 * @author munstfr
	 * @version 1.0
	 */
	@SuppressWarnings("PMD.AtLeastOneConstructor")
	private class MyFocusListener implements FocusListener {

		/**
		 * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
		 */
		@Override
		public void focusLost(final FocusEvent arg0) {
			// Update text info
			text = inputText.getText();
		}

		/**
		 * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
		 */
		@Override
		public void focusGained(final FocusEvent arg0) {
			// nothing to do
		}
	}

}
